<?php
/**
 * WooCommerce Bambora Gateway
 *
 * This source file is subject to the GNU General Public License v3.0
 * that is bundled with this package in the file license.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.gnu.org/licenses/gpl-3.0.html
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@skyverge.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade WooCommerce Bambora Gateway to newer
 * versions in the future. If you wish to customize WooCommerce Bambora Gateway for your
 * needs please refer to http://docs.woocommerce.com/document/bambora/
 *
 * @package   WooCommerceBambora/Gateway
 * @author    SkyVerge
 * @copyright Copyright (c) 2012-2024, SkyVerge, Inc. (info@skyverge.com)
 * @license   http://www.gnu.org/licenses/gpl-3.0.html GNU General Public License v3.0
 */

namespace SkyVerge\WooCommerce\Bambora\Gateway;

use SkyVerge\WooCommerce\PluginFramework\v5_12_1 as Framework;

defined( 'ABSPATH' ) or exit;

/**
 * Handles the custom Bambora payment form.
 *
 * This overrides the framework's implementation to support Bambora Custom Checkout,
 * which uses iframes for the card inputs like Braintree.
 *
 * @since 2.0.0
 *
 * @method Credit_Card get_gateway()
 */
class Payment_Form extends Framework\SV_WC_Payment_Gateway_Payment_Form {


	/** @var string JavaScript handler base class name */
	protected $js_handler_base_class_name = 'WC_Bambora_Payment_Form_Handler';


	/**
	 * Gets the JS handler class name.
	 *
	 * @since 2.2.8
	 *
	 * @return string
	 */
	protected function get_js_handler_class_name() {

		return $this->js_handler_base_class_name;
	}


	/**
	 * Renders the payment form description.
	 *
	 * Overridden to add the test card numbers for easier sandbox testing.
	 * @see Framework\SV_WC_Payment_Gateway_Payment_Form::render_payment_form_description()
	 *
	 * @since 2.0.0
	 */
	public function render_payment_form_description() {

		parent::render_payment_form_description();

		if ( $this->get_gateway()->is_test_environment() ) : ?>

			<p><?php esc_html_e( 'Test card numbers:', 'woocommerce-gateway-bambora' ); ?></p>

			<ul class="wc-bambora-test-card-numbers">

				<?php foreach ( $this->get_gateway()->get_test_cards() as $test_card_number => $test_card_label ) : ?>

					<li><code><?php echo esc_html( $test_card_number ); ?></code> - <?php echo esc_html( $test_card_label ); ?></li>

				<?php endforeach; ?>

			</ul>

		<?php endif;
	}


	/**
	 * Renders the payment fields.
	 *
	 * Overridden to add the hidden fields for Custom Checkout tokenization.
	 *
	 * @see Framework\SV_WC_Payment_Gateway_Payment_Form::render_payment_fields()
	 *
	 * @since 2.0.0
	 */
	public function render_payment_fields() {

		parent::render_payment_fields();

		$hidden_fields = array(
			'js-token',
			'account-number', // this will only be the last four, but name it so it's fully handled by the framework when setting order notes/meta
			'exp-month',
			'exp-year',
			'card-type',
		);

		foreach ( $hidden_fields as $field ) {
			echo '<input type="hidden" name="wc-' . $this->get_gateway()->get_id_dasherized() . '-' . sanitize_html_class( $field ) . '" />';
		}
	}


	/**
	 * Renders the payment fields.
	 *
	 * Overridden to output empty divs for each field instead of actual inputs.
	 * Bambora Custom Checkout uses these to attach iframe inputs.
	 * The divs are styled like the inputs would be, and the transparent iframes sit inside.
	 *
	 * @since 2.0.0
	 *
	 * @param array $field payment field params
	 */
	public function render_payment_field( $field ) {

		?>
		<div class="form-row <?php echo implode( ' ', array_map( 'sanitize_html_class', isset( $field['class'] ) ? (array) $field['class'] : [] ) ); ?>">
			<label for="<?php echo esc_attr( $field['id'] ) . '-hosted'; ?>"><?php echo esc_html( $field['label'] ?? '' ); if ( ! empty( $field['required'] ) ) : ?><abbr class="required" title="required">&nbsp;*</abbr><?php endif; ?></label>
			<div id="<?php echo esc_attr( $field['id'] ) . '-hosted'; ?>" class="<?php echo implode( ' ', array_map( 'sanitize_html_class', isset( $field['input_class'] ) ? (array) $field['input_class'] : [] ) ); ?>" data-placeholder="<?php echo isset( $field['placeholder'] ) ? esc_attr( $field['placeholder'] ) : ''; ?>"></div>
		</div>
		<?php
	}


	/**
	 * Gets the JS args for the payment form handler.
	 *
	 * @since 2.2.7
	 *
	 * @return array
	 */
	protected function get_js_handler_args() {

		return [
			'id'         => $this->get_gateway()->get_id(),
			'slug'       => $this->get_gateway()->get_id_dasherized(),
			'debug'      => $this->get_gateway()->debug_log(),
			'card_types' => $this->get_available_card_types(),
			'styles'     => $this->get_field_styles(),
		];
	}


	/**
	 * Gets the card types made available at checkout.
	 *
	 * This is configured in the plugin settings. Normally that setting doesn't
	 * affect which card types are allow through, but Bambora Custom Checkout
	 * allows us to pass card types which is used during form validation and will
	 * throw an error if an unsupported type is used.
	 *
	 * Note that we adjust at least one card type slug to match what their JS
	 * expects.
	 *
	 * @since 2.0.0
	 *
	 * @return array
	 */
	protected function get_available_card_types() {

		$types = array_map( array( '\\SkyVerge\\WooCommerce\\PluginFramework\\v5_12_1\\SV_WC_Payment_Gateway_Helper', 'normalize_card_type' ), $this->get_gateway()->get_card_types() );

		$types = str_replace( Framework\SV_WC_Payment_Gateway_Helper::CARD_TYPE_DINERSCLUB, 'diners', $types );

		return $types;
	}


	/**
	 * Gets the payment field styles.
	 *
	 * Their JS allows us to specify styles to render on each iframe field.
	 * This sets some common base styles for each, and allows filtering to further match other theme-specific styles.
	 *
	 * @since 2.0.0
	 *
	 * @return array<string, mixed>
	 */
	protected function get_field_styles() : array {

		return $this->get_gateway()->get_checkout_field_styles();
	}


}
